@@ -1,6 +1,7 @@
 #ifndef MUDPF_FITZ_H
 #define MUDPF_FITZ_H
 
+#include "mupdf/fitz/version.h"
 #include "mupdf/fitz/system.h"
 #include "mupdf/fitz/context.h"
 
@@ -9,6 +10,7 @@
 #include "mupdf/fitz/hash.h"
 #include "mupdf/fitz/math.h"
 #include "mupdf/fitz/string.h"
+#include "mupdf/fitz/tree.h"
 #include "mupdf/fitz/xml.h"
 
 /* I/O */
@@ -17,11 +19,13 @@
 #include "mupdf/fitz/compressed-buffer.h"
 #include "mupdf/fitz/filter.h"
 #include "mupdf/fitz/output.h"
+#include "mupdf/fitz/unzip.h"
 
 /* Resources */
 #include "mupdf/fitz/store.h"
 #include "mupdf/fitz/colorspace.h"
 #include "mupdf/fitz/pixmap.h"
+#include "mupdf/fitz/glyph.h"
 #include "mupdf/fitz/bitmap.h"
 #include "mupdf/fitz/image.h"
 #include "mupdf/fitz/function.h"
@@ -42,7 +46,6 @@
 #include "mupdf/fitz/outline.h"
 #include "mupdf/fitz/document.h"
 #include "mupdf/fitz/annotation.h"
-#include "mupdf/fitz/meta.h"
 
 #include "mupdf/fitz/write-document.h"
 
@@ -52,5 +55,6 @@
 #include "mupdf/fitz/output-pwg.h"
 #include "mupdf/fitz/output-pcl.h"
 #include "mupdf/fitz/output-svg.h"
+#include "mupdf/fitz/output-tga.h"
 
 #endif
