@@ -76,6 +76,6 @@ struct fz_compressed_buffer_s
 	fz_buffer *buffer;
 };
 
-void fz_free_compressed_buffer(fz_context *ctx, fz_compressed_buffer *buf);
+void fz_drop_compressed_buffer(fz_context *ctx, fz_compressed_buffer *buf);
 
 #endif
