@@ -20,7 +20,7 @@ void fz_write_png(fz_context *ctx, fz_pixmap *pixmap, char *filename, int saveal
 /*
 	Output a pixmap to an output stream as a png.
 */
-void fz_output_png(fz_output *out, const fz_pixmap *pixmap, int savealpha);
+void fz_output_png(fz_context *ctx, fz_output *out, const fz_pixmap *pixmap, int savealpha);
 
 /*
 	Get an image as a png in a buffer.
@@ -31,10 +31,8 @@ fz_buffer *fz_new_png_from_pixmap(fz_context *ctx, fz_pixmap *pixmap);
 
 typedef struct fz_png_output_context_s fz_png_output_context;
 
-fz_png_output_context *fz_output_png_header(fz_output *out, int w, int h, int n, int savealpha);
-
-void fz_output_png_band(fz_output *out, int w, int h, int n, int band, int bandheight, unsigned char *samples, int savealpha, fz_png_output_context *poc);
-
-void fz_output_png_trailer(fz_output *out, fz_png_output_context *poc);
+fz_png_output_context *fz_output_png_header(fz_context *ctx, fz_output *out, int w, int h, int n, int savealpha);
+void fz_output_png_band(fz_context *ctx, fz_output *out, int w, int h, int n, int band, int bandheight, unsigned char *samples, int savealpha, fz_png_output_context *poc);
+void fz_output_png_trailer(fz_context *ctx, fz_output *out, fz_png_output_context *poc);
 
 #endif
