@@ -3,6 +3,7 @@
 
 #include "mupdf/fitz.h"
 
+#include "mupdf/pdf/name-table.h"
 #include "mupdf/pdf/object.h"
 #include "mupdf/pdf/document.h"
 #include "mupdf/pdf/parse.h"
@@ -13,6 +14,7 @@
 #include "mupdf/pdf/resource.h"
 #include "mupdf/pdf/cmap.h"
 #include "mupdf/pdf/font.h"
+#include "mupdf/pdf/interpret.h"
 
 #include "mupdf/pdf/annot.h"
 #include "mupdf/pdf/field.h"
@@ -23,4 +25,6 @@
 
 #include "mupdf/pdf/output-pdf.h"
 
+#include "mupdf/pdf/clean.h"
+
 #endif
