@@ -58,6 +58,8 @@ public class ReaderView
 	private final Stepper     mStepper;
 	private int               mScrollerLastX;
 	private int               mScrollerLastY;
+	private float		  mLastScaleFocusX;
+	private float		  mLastScaleFocusY;
 
 	static abstract class ViewMapper {
 		abstract void applyToView(View view);
@@ -408,7 +410,7 @@ public class ReaderView
 				}
 				break;
 			case MOVING_DOWN:
-				if (HORIZONTAL_SCROLLING && bounds.bottom <= 0) {
+				if (!HORIZONTAL_SCROLLING && bounds.bottom <= 0) {
 					// Fling off to the bottom bring previous view onto screen
 					View vr = mChildViews.get(mCurrent-1);
 
@@ -478,12 +480,22 @@ public class ReaderView
 
 			View v = mChildViews.get(mCurrent);
 			if (v != null) {
+				float currentFocusX = detector.getFocusX();
+				float currentFocusY = detector.getFocusY();
 				// Work out the focus point relative to the view top left
-				int viewFocusX = (int)detector.getFocusX() - (v.getLeft() + mXScroll);
-				int viewFocusY = (int)detector.getFocusY() - (v.getTop() + mYScroll);
+				int viewFocusX = (int)currentFocusX - (v.getLeft() + mXScroll);
+				int viewFocusY = (int)currentFocusY - (v.getTop() + mYScroll);
 				// Scroll to maintain the focus point
 				mXScroll += viewFocusX - viewFocusX * factor;
 				mYScroll += viewFocusY - viewFocusY * factor;
+
+				if (mLastScaleFocusX>=0)
+					mXScroll+=currentFocusX-mLastScaleFocusX;
+				if (mLastScaleFocusY>=0)
+					mYScroll+=currentFocusY-mLastScaleFocusY;
+
+				mLastScaleFocusX=currentFocusX;
+				mLastScaleFocusY=currentFocusY;
 				requestLayout();
 			}
 		}
@@ -496,6 +508,7 @@ public class ReaderView
 		// screen is not showing the effect of them, so they can
 		// only confuse the user
 		mXScroll = mYScroll = 0;
+		mLastScaleFocusX = mLastScaleFocusY = -1;
 		return true;
 	}
 
@@ -539,7 +552,6 @@ public class ReaderView
 			}
 		}
 
-		requestLayout();
 		return true;
 	}
 
