@@ -3,12 +3,8 @@
 #import <Foundation/Foundation.h>
 
 @interface MuAnnotation : NSObject
-{
-	int type;
-	CGRect rect;
-}
--(id) initFromAnnot:(fz_annot *)annot forDoc:(fz_document *)doc;
+-(id) initFromAnnot:(fz_annot *)annot forPage:(fz_page *)page;
 @property(readonly) int type;
 @property(readonly) CGRect rect;
-+(MuAnnotation *) annotFromAnnot:(fz_annot *)annot forDoc:(fz_document *)doc;
++(MuAnnotation *) annotFromAnnot:(fz_annot *)annot forPage:(fz_page *)page;
 @end
