@@ -23,17 +23,20 @@
 			}
 			else
 			{
-				pdf_document *idoc = pdf_specifics(doc);
-				if (idoc) pdf_enable_js(idoc);
-				interactive = (idoc != NULL) && (pdf_crypt_version(idoc) == 0);
+				pdf_document *idoc = pdf_specifics(ctx, doc);
+				if (idoc) pdf_enable_js(ctx, idoc);
+				interactive = (idoc != NULL) && (pdf_crypt_version(ctx, idoc) == 0);
 			}
 		}
 		fz_catch(ctx)
 		{
-			if (doc != NULL)
-				fz_close_document(doc);
-			[self release];
-			self = nil;
+			if (self)
+			{
+				if (doc != NULL)
+					fz_drop_document(ctx, doc);
+				[self release];
+				self = nil;
+			}
 		}
 	}
 	return self;
@@ -43,7 +46,7 @@
 {
 	__block fz_document *block_doc = doc;
 	dispatch_async(queue, ^{
-		fz_close_document(block_doc);
+		fz_drop_document(ctx, block_doc);
 	});
 	[super dealloc];
 }
