@@ -152,12 +152,6 @@ void windocopy(pdfapp_t *app)
 {
 }
 
-void winreloadfile(pdfapp_t *app)
-{
-	pdfapp_close(app);
-	pdfapp_open(app, filename, 1);
-}
-
 void winreloadpage(pdfapp_t *app)
 {
 }
@@ -426,7 +420,7 @@ main(int argc, char *argv[])
 	if (file_open)
 		pdfapp_close(&gapp);
 
-	fz_free_context(ctx);
+	fz_drop_context(ctx);
 
 	return 0;
 }
