@@ -44,12 +44,7 @@ fz_buffer *
 fz_keep_buffer(fz_context *ctx, fz_buffer *buf)
 {
 	if (buf)
-	{
-		if (buf->refs == 1 && buf->cap > buf->len+1)
-			fz_resize_buffer(ctx, buf, buf->len);
 		buf->refs ++;
-	}
-
 	return buf;
 }
 
@@ -135,7 +130,7 @@ void fz_write_buffer(fz_context *ctx, fz_buffer *buf, const void *data, int len)
 
 void fz_write_buffer_byte(fz_context *ctx, fz_buffer *buf, int val)
 {
-	if (buf->len > buf->cap)
+	if (buf->len + 1 > buf->cap)
 		fz_grow_buffer(ctx, buf);
 	buf->data[buf->len++] = val;
 	buf->unused_bits = 0;
@@ -366,7 +361,7 @@ fz_test_buffer_write(fz_context *ctx)
 			k = (rand() & 31)+1;
 			if (k > j)
 				k = j;
-			fz_write_buffer_bits(ctx, copy, fz_read_bits(stm, k), k);
+			fz_write_buffer_bits(ctx, copy, fz_read_bits(ctx, stm, k), k);
 			j -= k;
 		}
 		while (j);
@@ -375,7 +370,7 @@ fz_test_buffer_write(fz_context *ctx)
 			fprintf(stderr, "Copied buffer is different!\n");
 		fz_seek(stm, 0, 0);
 	}
-	fz_close(stm);
+	fz_drop_stream(stm);
 	fz_drop_buffer(ctx, master);
 	fz_drop_buffer(ctx, copy);
 }
