@@ -7,4 +7,6 @@ void fz_register_document_handlers(fz_context *ctx)
 	fz_register_document_handler(ctx, &cbz_document_handler);
 	fz_register_document_handler(ctx, &img_document_handler);
 	fz_register_document_handler(ctx, &tiff_document_handler);
+	fz_register_document_handler(ctx, &html_document_handler);
+	fz_register_document_handler(ctx, &epub_document_handler);
 }
