@@ -45,7 +45,7 @@ fz_drop_function(fz_context *ctx, fz_function *func)
 }
 
 unsigned int
-fz_function_size(fz_function *func)
+fz_function_size(fz_context *ctx, fz_function *func)
 {
 	return (func ? func->size : 0);
 }
