@@ -1,5 +1,11 @@
 #include "mupdf/fitz.h"
 
+/* This definition will be made elsewhere soon, but putting it here
+ * temporarily means the commits can be sensibly ordered. */
+typedef int fz_off_t;
+
+static const char *fz_hex_digits = "0123456789abcdef";
+
 struct fmtbuf
 {
 	char *p;
@@ -14,44 +20,6 @@ static void fmtputc(struct fmtbuf *out, int c)
 	++(out->n);
 }
 
-/*
- * Compute decimal integer m, exp such that:
- *	f = m*10^exp
- *	m is as short as possible with losing exactness
- * assumes special cases (NaN, +Inf, -Inf) have been handled.
- */
-static void fz_dtoa(float f, char *digits, int *exp, int *neg, int *ndigits)
-{
-	char buf[20], *s, *p, *e;
-	int n;
-
-	/* TODO: binary search */
-	for (n = 1; n < 9; ++n) {
-		sprintf(buf, "%+.*e", n, f);
-		if (strtof(buf, NULL) == f)
-			break;
-	}
-
-	*neg = (buf[0] == '-');
-
-	p = buf + 3;
-	e = strchr(p, 'e');
-	*exp = atoi(e + 1) - (e - p);
-
-	if (e[-1] == '0') {
-		--e;
-		++(*exp);
-	}
-
-	s = digits;
-	*s++ = buf[1];
-	while (p < e)
-		*s++ = *p++;
-	*s = 0;
-
-	*ndigits = s - digits;
-}
-
 /*
  * Convert float to shortest possible string that won't lose precision, except:
  * NaN to 0, +Inf to FLT_MAX, -Inf to -FLT_MAX.
@@ -64,7 +32,7 @@ static void fmtfloat(struct fmtbuf *out, float f)
 	if (isnan(f)) f = 0;
 	if (isinf(f)) f = f < 0 ? -FLT_MAX : FLT_MAX;
 
-	fz_dtoa(f, digits, &exp, &neg, &ndigits);
+	fz_ftoa(f, digits, &exp, &neg, &ndigits);
 	point = exp + ndigits;
 
 	if (neg)
@@ -92,12 +60,41 @@ static void fmtfloat(struct fmtbuf *out, float f)
 	}
 }
 
+static void fmtuint(struct fmtbuf *out, unsigned int a, int z, int base)
+{
+	char buf[40];
+	int i;
+
+	i = 0;
+	while (a) {
+		buf[i++] = fz_hex_digits[a % base];
+		a /= base;
+	}
+	while (i < z)
+		buf[i++] = '0';
+	while (i > 0)
+		fmtputc(out, buf[--i]);
+}
+
+static void fmtuint64(struct fmtbuf *out, uint64_t a, int z, int base)
+{
+	char buf[80];
+	int i;
+
+	i = 0;
+	while (a) {
+		buf[i++] = fz_hex_digits[a % base];
+		a /= base;
+	}
+	while (i < z)
+		buf[i++] = '0';
+	while (i > 0)
+		fmtputc(out, buf[--i]);
+}
+
 static void fmtint(struct fmtbuf *out, int value, int z, int base)
 {
-	static const char *digits = "0123456789abcdef";
-	char buf[40];
 	unsigned int a;
-	int i;
 
 	if (value < 0)
 	{
@@ -106,16 +103,21 @@ static void fmtint(struct fmtbuf *out, int value, int z, int base)
 	}
 	else
 		a = value;
+	fmtuint(out, a, z, base);
+}
 
-	i = 0;
-	while (a) {
-		buf[i++] = digits[a % base];
-		a /= base;
+static void fmtint64(struct fmtbuf *out, int64_t value, int z, int base)
+{
+	unsigned int a;
+
+	if (value < 0)
+	{
+		fmtputc(out, '-');
+		a = -value;
 	}
-	while (i < z)
-		buf[i++] = '0';
-	while (i > 0)
-		fmtputc(out, buf[--i]);
+	else
+		a = value;
+	fmtuint64(out, a, z, base);
 }
 
 static void fmtquote(struct fmtbuf *out, const char *s, int sq, int eq)
@@ -155,8 +157,10 @@ fz_vsnprintf(char *buffer, int space, const char *fmt, va_list args)
 	fz_rect *r;
 	fz_point *p;
 	int c, i, n, z;
+	int64_t i64;
 	double f;
 	char *s;
+	int length;
 
 	out.p = buffer;
 	out.s = space;
@@ -172,6 +176,38 @@ fz_vsnprintf(char *buffer, int space, const char *fmt, va_list args)
 			if (c == '0' && fmt[0] && fmt[1]) {
 				z = *fmt++ - '0';
 				c = *fmt++;
+				while (c >= '0' && c <= '9' && fmt[0])
+				{
+					z = z*10 + c - '0';
+					c = *fmt++;
+				}
+			}
+			/* Check for lengths */
+			length = 0;
+			switch (c) {
+			case 'l':
+				c = *fmt++;
+				if (c == 'l')
+					length = 64;
+				else
+					fmt--;
+				break;
+			case 'z':
+				if (sizeof(size_t) >= 8)
+					length = 64;
+				break;
+			case 'Z':
+				if (sizeof(fz_off_t) >= 8)
+					length = 64;
+				else
+					length = 32;
+				break;
+			}
+			if (length != 0)
+			{
+				c = *fmt++;
+				if (c == 0)
+					break; /* Can't warn :( */
 			}
 			switch (c) {
 			default:
@@ -181,7 +217,7 @@ fz_vsnprintf(char *buffer, int space, const char *fmt, va_list args)
 			case '%':
 				fmtputc(&out, '%');
 				break;
-			case 'M':
+			case 'M': /* fz_matrix * */
 				m = va_arg(args, fz_matrix*);
 				fmtfloat(&out, m->a); fmtputc(&out, ' ');
 				fmtfloat(&out, m->b); fmtputc(&out, ' ');
@@ -190,19 +226,19 @@ fz_vsnprintf(char *buffer, int space, const char *fmt, va_list args)
 				fmtfloat(&out, m->e); fmtputc(&out, ' ');
 				fmtfloat(&out, m->f);
 				break;
-			case 'R':
+			case 'R': /* fz_rect * */
 				r = va_arg(args, fz_rect*);
 				fmtfloat(&out, r->x0); fmtputc(&out, ' ');
 				fmtfloat(&out, r->y0); fmtputc(&out, ' ');
 				fmtfloat(&out, r->x1); fmtputc(&out, ' ');
 				fmtfloat(&out, r->y1);
 				break;
-			case 'P':
+			case 'P': /* fz_point * */
 				p = va_arg(args, fz_point*);
 				fmtfloat(&out, p->x); fmtputc(&out, ' ');
 				fmtfloat(&out, p->y);
 				break;
-			case 'C':
+			case 'C': /* unicode char */
 				c = va_arg(args, int);
 				if (c < 128)
 					fmtputc(&out, c);
@@ -223,12 +259,40 @@ fz_vsnprintf(char *buffer, int space, const char *fmt, va_list args)
 				fmtfloat(&out, f);
 				break;
 			case 'x':
-				i = va_arg(args, int);
-				fmtint(&out, i, z, 16);
+				if (length == 64)
+				{
+					i64 = va_arg(args, int64_t);
+					fmtuint64(&out, i64, z, 16);
+				}
+				else
+				{
+					i = va_arg(args, int);
+					fmtuint(&out, i, z, 16);
+				}
 				break;
 			case 'd':
-				i = va_arg(args, int);
-				fmtint(&out, i, z, 10);
+				if (length == 64)
+				{
+					i64 = va_arg(args, int64_t);
+					fmtint64(&out, i64, z, 10);
+				}
+				else
+				{
+					i = va_arg(args, int);
+					fmtint(&out, i, z, 10);
+				}
+				break;
+			case 'u':
+				if (length == 64)
+				{
+					i64 = va_arg(args, int64_t);
+					fmtuint64(&out, i64, z, 10);
+				}
+				else
+				{
+					i = va_arg(args, int);
+					fmtuint(&out, i, z, 10);
+				}
 				break;
 			case 'o':
 				i = va_arg(args, int);
@@ -290,3 +354,25 @@ fz_vfprintf(fz_context *ctx, FILE *file, const char *fmt, va_list old_args)
 
 	return l;
 }
+
+int
+fz_fprintf(fz_context *ctx, FILE *file, const char *fmt, ...)
+{
+	int n;
+	va_list ap;
+	va_start(ap, fmt);
+	n = fz_vfprintf(ctx, file, fmt, ap);
+	va_end(ap);
+	return n;
+}
+
+int
+fz_snprintf(char *buffer, int space, const char *fmt, ...)
+{
+	int n;
+	va_list ap;
+	va_start(ap, fmt);
+	n = fz_vsnprintf(buffer, space, fmt, ap);
+	va_end(ap);
+	return n;
+}
