@@ -35,6 +35,6 @@ void jsB_initboolean(js_State *J)
 		jsB_propf(J, "toString", Bp_toString, 0);
 		jsB_propf(J, "valueOf", Bp_valueOf, 0);
 	}
-	js_newcconstructor(J, jsB_Boolean, jsB_new_Boolean, 1);
+	js_newcconstructor(J, jsB_Boolean, jsB_new_Boolean, "Boolean", 1);
 	js_defglobal(J, "Boolean", JS_DONTENUM);
 }
