@@ -15,8 +15,6 @@
 #define STM3(x,a,b,c)	jsP_newnode(J, STM_ ## x, a, b, c, 0)
 #define STM4(x,a,b,c,d)	jsP_newnode(J, STM_ ## x, a, b, c, d)
 
-#define TOKSTR		jsY_tokenstring(J->lookahead)
-
 static js_Ast *expression(js_State *J, int notin);
 static js_Ast *assignment(js_State *J, int notin);
 static js_Ast *memberexp(js_State *J);
@@ -57,7 +55,7 @@ static js_Ast *jsP_newnode(js_State *J, int type, js_Ast *a, js_Ast *b, js_Ast *
 	js_Ast *node = js_malloc(J, sizeof *node);
 
 	node->type = type;
-	node->line = J->lexline;
+	node->line = J->astline;
 	node->a = a;
 	node->b = b;
 	node->c = c;
@@ -128,36 +126,25 @@ void jsP_freeparse(js_State *J)
 
 /* Lookahead */
 
-static void next(js_State *J)
+static void jsP_next(js_State *J)
 {
+	J->astline = J->lexline;
 	J->lookahead = jsY_lex(J);
 }
 
-static int accept(js_State *J, int t)
-{
-	if (J->lookahead == t) {
-		next(J);
-		return 1;
-	}
-	return 0;
-}
+#define jsP_accept(J,x) (J->lookahead == x ? (jsP_next(J), 1) : 0)
 
-static void expect(js_State *J, int t)
-{
-	if (accept(J, t))
-		return;
-	jsP_error(J, "unexpected token: %s (expected %s)", TOKSTR, jsY_tokenstring(t));
-}
+#define jsP_expect(J,x) if (!jsP_accept(J, x)) jsP_error(J, "unexpected token: %s (expected %s)", jsY_tokenstring(J->lookahead), jsY_tokenstring(x))
 
 static void semicolon(js_State *J)
 {
 	if (J->lookahead == ';') {
-		next(J);
+		jsP_next(J);
 		return;
 	}
 	if (J->newline || J->lookahead == '}' || J->lookahead == 0)
 		return;
-	jsP_error(J, "unexpected token: %s (expected ';')", TOKSTR);
+	jsP_error(J, "unexpected token: %s (expected ';')", jsY_tokenstring(J->lookahead));
 }
 
 /* Literals */
@@ -185,10 +172,10 @@ static js_Ast *identifier(js_State *J)
 	if (J->lookahead == TK_IDENTIFIER) {
 		checkfutureword(J, J->text);
 		a = jsP_newstrnode(J, AST_IDENTIFIER, J->text);
-		next(J);
+		jsP_next(J);
 		return a;
 	}
-	jsP_error(J, "unexpected token: %s (expected identifier)", TOKSTR);
+	jsP_error(J, "unexpected token: %s (expected identifier)", jsY_tokenstring(J->lookahead));
 }
 
 static js_Ast *identifieropt(js_State *J)
@@ -202,10 +189,10 @@ static js_Ast *identifiername(js_State *J)
 {
 	if (J->lookahead == TK_IDENTIFIER || J->lookahead >= TK_BREAK) {
 		js_Ast *a = jsP_newstrnode(J, AST_IDENTIFIER, J->text);
-		next(J);
+		jsP_next(J);
 		return a;
 	}
-	jsP_error(J, "unexpected token: %s (expected identifier or keyword)", TOKSTR);
+	jsP_error(J, "unexpected token: %s (expected identifier or keyword)", jsY_tokenstring(J->lookahead));
 }
 
 static js_Ast *arrayelement(js_State *J)
@@ -221,7 +208,7 @@ static js_Ast *arrayliteral(js_State *J)
 	if (J->lookahead == ']')
 		return NULL;
 	head = tail = LIST(arrayelement(J));
-	while (accept(J, ',')) {
+	while (jsP_accept(J, ',')) {
 		if (J->lookahead != ']')
 			tail = tail->b = LIST(arrayelement(J));
 	}
@@ -233,10 +220,10 @@ static js_Ast *propname(js_State *J)
 	js_Ast *name;
 	if (J->lookahead == TK_NUMBER) {
 		name = jsP_newnumnode(J, EXP_NUMBER, J->number);
-		next(J);
+		jsP_next(J);
 	} else if (J->lookahead == TK_STRING) {
 		name = jsP_newstrnode(J, EXP_STRING, J->text);
-		next(J);
+		jsP_next(J);
 	} else {
 		name = identifiername(J);
 	}
@@ -252,22 +239,22 @@ static js_Ast *propassign(js_State *J)
 	if (J->lookahead != ':' && name->type == AST_IDENTIFIER) {
 		if (!strcmp(name->string, "get")) {
 			name = propname(J);
-			expect(J, '(');
-			expect(J, ')');
+			jsP_expect(J, '(');
+			jsP_expect(J, ')');
 			body = funbody(J);
 			return EXP3(PROP_GET, name, NULL, body);
 		}
 		if (!strcmp(name->string, "set")) {
 			name = propname(J);
-			expect(J, '(');
+			jsP_expect(J, '(');
 			arg = identifier(J);
-			expect(J, ')');
+			jsP_expect(J, ')');
 			body = funbody(J);
 			return EXP3(PROP_SET, name, LIST(arg), body);
 		}
 	}
 
-	expect(J, ':');
+	jsP_expect(J, ':');
 	value = assignment(J, 0);
 	return EXP2(PROP_VAL, name, value);
 }
@@ -278,7 +265,7 @@ static js_Ast *objectliteral(js_State *J)
 	if (J->lookahead == '}')
 		return NULL;
 	head = tail = LIST(propassign(J));
-	while (accept(J, ',')) {
+	while (jsP_accept(J, ',')) {
 		if (J->lookahead == '}')
 			break;
 		tail = tail->b = LIST(propassign(J));
@@ -294,7 +281,7 @@ static js_Ast *parameters(js_State *J)
 	if (J->lookahead == ')')
 		return NULL;
 	head = tail = LIST(identifier(J));
-	while (accept(J, ',')) {
+	while (jsP_accept(J, ',')) {
 		tail = tail->b = LIST(identifier(J));
 	}
 	return jsP_list(head);
@@ -304,9 +291,9 @@ static js_Ast *fundec(js_State *J)
 {
 	js_Ast *a, *b, *c;
 	a = identifier(J);
-	expect(J, '(');
+	jsP_expect(J, '(');
 	b = parameters(J);
-	expect(J, ')');
+	jsP_expect(J, ')');
 	c = funbody(J);
 	return jsP_newnode(J, AST_FUNDEC, a, b, c, 0);
 }
@@ -315,9 +302,9 @@ static js_Ast *funstm(js_State *J)
 {
 	js_Ast *a, *b, *c;
 	a = identifier(J);
-	expect(J, '(');
+	jsP_expect(J, '(');
 	b = parameters(J);
-	expect(J, ')');
+	jsP_expect(J, ')');
 	c = funbody(J);
 	/* rewrite function statement as "var X = function X() {}" */
 	return STM1(VAR, LIST(EXP2(VAR, a, EXP3(FUN, a, b, c))));
@@ -327,9 +314,9 @@ static js_Ast *funexp(js_State *J)
 {
 	js_Ast *a, *b, *c;
 	a = identifieropt(J);
-	expect(J, '(');
+	jsP_expect(J, '(');
 	b = parameters(J);
-	expect(J, ')');
+	jsP_expect(J, ')');
 	c = funbody(J);
 	return EXP3(FUN, a, b, c);
 }
@@ -343,35 +330,35 @@ static js_Ast *primary(js_State *J)
 	if (J->lookahead == TK_IDENTIFIER) {
 		checkfutureword(J, J->text);
 		a = jsP_newstrnode(J, EXP_IDENTIFIER, J->text);
-		next(J);
+		jsP_next(J);
 		return a;
 	}
 	if (J->lookahead == TK_STRING) {
 		a = jsP_newstrnode(J, EXP_STRING, J->text);
-		next(J);
+		jsP_next(J);
 		return a;
 	}
 	if (J->lookahead == TK_REGEXP) {
 		a = jsP_newstrnode(J, EXP_REGEXP, J->text);
 		a->number = J->number;
-		next(J);
+		jsP_next(J);
 		return a;
 	}
 	if (J->lookahead == TK_NUMBER) {
 		a = jsP_newnumnode(J, EXP_NUMBER, J->number);
-		next(J);
+		jsP_next(J);
 		return a;
 	}
 
-	if (accept(J, TK_THIS)) return EXP0(THIS);
-	if (accept(J, TK_NULL)) return EXP0(NULL);
-	if (accept(J, TK_TRUE)) return EXP0(TRUE);
-	if (accept(J, TK_FALSE)) return EXP0(FALSE);
-	if (accept(J, '{')) { a = EXP1(OBJECT, objectliteral(J)); expect(J, '}'); return a; }
-	if (accept(J, '[')) { a = EXP1(ARRAY, arrayliteral(J)); expect(J, ']'); return a; }
-	if (accept(J, '(')) { a = expression(J, 0); expect(J, ')'); return a; }
+	if (jsP_accept(J, TK_THIS)) return EXP0(THIS);
+	if (jsP_accept(J, TK_NULL)) return EXP0(NULL);
+	if (jsP_accept(J, TK_TRUE)) return EXP0(TRUE);
+	if (jsP_accept(J, TK_FALSE)) return EXP0(FALSE);
+	if (jsP_accept(J, '{')) { a = EXP1(OBJECT, objectliteral(J)); jsP_expect(J, '}'); return a; }
+	if (jsP_accept(J, '[')) { a = EXP1(ARRAY, arrayliteral(J)); jsP_expect(J, ']'); return a; }
+	if (jsP_accept(J, '(')) { a = expression(J, 0); jsP_expect(J, ')'); return a; }
 
-	jsP_error(J, "unexpected token in expression: %s", TOKSTR);
+	jsP_error(J, "unexpected token in expression: %s", jsY_tokenstring(J->lookahead));
 }
 
 static js_Ast *arguments(js_State *J)
@@ -380,7 +367,7 @@ static js_Ast *arguments(js_State *J)
 	if (J->lookahead == ')')
 		return NULL;
 	head = tail = LIST(assignment(J, 0));
-	while (accept(J, ',')) {
+	while (jsP_accept(J, ',')) {
 		tail = tail->b = LIST(assignment(J, 0));
 	}
 	return jsP_list(head);
@@ -390,17 +377,17 @@ static js_Ast *newexp(js_State *J)
 {
 	js_Ast *a, *b;
 
-	if (accept(J, TK_NEW)) {
+	if (jsP_accept(J, TK_NEW)) {
 		a = memberexp(J);
-		if (accept(J, '(')) {
+		if (jsP_accept(J, '(')) {
 			b = arguments(J);
-			expect(J, ')');
+			jsP_expect(J, ')');
 			return EXP2(NEW, a, b);
 		}
 		return EXP1(NEW, a);
 	}
 
-	if (accept(J, TK_FUNCTION))
+	if (jsP_accept(J, TK_FUNCTION))
 		return funexp(J);
 
 	return primary(J);
@@ -410,8 +397,8 @@ static js_Ast *memberexp(js_State *J)
 {
 	js_Ast *a = newexp(J);
 loop:
-	if (accept(J, '.')) { a = EXP2(MEMBER, a, identifiername(J)); goto loop; }
-	if (accept(J, '[')) { a = EXP2(INDEX, a, expression(J, 0)); expect(J, ']'); goto loop; }
+	if (jsP_accept(J, '.')) { a = EXP2(MEMBER, a, identifiername(J)); goto loop; }
+	if (jsP_accept(J, '[')) { a = EXP2(INDEX, a, expression(J, 0)); jsP_expect(J, ']'); goto loop; }
 	return a;
 }
 
@@ -419,31 +406,31 @@ static js_Ast *callexp(js_State *J)
 {
 	js_Ast *a = newexp(J);
 loop:
-	if (accept(J, '.')) { a = EXP2(MEMBER, a, identifiername(J)); goto loop; }
-	if (accept(J, '[')) { a = EXP2(INDEX, a, expression(J, 0)); expect(J, ']'); goto loop; }
-	if (accept(J, '(')) { a = EXP2(CALL, a, arguments(J)); expect(J, ')'); goto loop; }
+	if (jsP_accept(J, '.')) { a = EXP2(MEMBER, a, identifiername(J)); goto loop; }
+	if (jsP_accept(J, '[')) { a = EXP2(INDEX, a, expression(J, 0)); jsP_expect(J, ']'); goto loop; }
+	if (jsP_accept(J, '(')) { a = EXP2(CALL, a, arguments(J)); jsP_expect(J, ')'); goto loop; }
 	return a;
 }
 
 static js_Ast *postfix(js_State *J)
 {
 	js_Ast *a = callexp(J);
-	if (!J->newline && accept(J, TK_INC)) return EXP1(POSTINC, a);
-	if (!J->newline && accept(J, TK_DEC)) return EXP1(POSTDEC, a);
+	if (!J->newline && jsP_accept(J, TK_INC)) return EXP1(POSTINC, a);
+	if (!J->newline && jsP_accept(J, TK_DEC)) return EXP1(POSTDEC, a);
 	return a;
 }
 
 static js_Ast *unary(js_State *J)
 {
-	if (accept(J, TK_DELETE)) return EXP1(DELETE, unary(J));
-	if (accept(J, TK_VOID)) return EXP1(VOID, unary(J));
-	if (accept(J, TK_TYPEOF)) return EXP1(TYPEOF, unary(J));
-	if (accept(J, TK_INC)) return EXP1(PREINC, unary(J));
-	if (accept(J, TK_DEC)) return EXP1(PREDEC, unary(J));
-	if (accept(J, '+')) return EXP1(POS, unary(J));
-	if (accept(J, '-')) return EXP1(NEG, unary(J));
-	if (accept(J, '~')) return EXP1(BITNOT, unary(J));
-	if (accept(J, '!')) return EXP1(LOGNOT, unary(J));
+	if (jsP_accept(J, TK_DELETE)) return EXP1(DELETE, unary(J));
+	if (jsP_accept(J, TK_VOID)) return EXP1(VOID, unary(J));
+	if (jsP_accept(J, TK_TYPEOF)) return EXP1(TYPEOF, unary(J));
+	if (jsP_accept(J, TK_INC)) return EXP1(PREINC, unary(J));
+	if (jsP_accept(J, TK_DEC)) return EXP1(PREDEC, unary(J));
+	if (jsP_accept(J, '+')) return EXP1(POS, unary(J));
+	if (jsP_accept(J, '-')) return EXP1(NEG, unary(J));
+	if (jsP_accept(J, '~')) return EXP1(BITNOT, unary(J));
+	if (jsP_accept(J, '!')) return EXP1(LOGNOT, unary(J));
 	return postfix(J);
 }
 
@@ -451,9 +438,9 @@ static js_Ast *multiplicative(js_State *J)
 {
 	js_Ast *a = unary(J);
 loop:
-	if (accept(J, '*')) { a = EXP2(MUL, a, unary(J)); goto loop; }
-	if (accept(J, '/')) { a = EXP2(DIV, a, unary(J)); goto loop; }
-	if (accept(J, '%')) { a = EXP2(MOD, a, unary(J)); goto loop; }
+	if (jsP_accept(J, '*')) { a = EXP2(MUL, a, unary(J)); goto loop; }
+	if (jsP_accept(J, '/')) { a = EXP2(DIV, a, unary(J)); goto loop; }
+	if (jsP_accept(J, '%')) { a = EXP2(MOD, a, unary(J)); goto loop; }
 	return a;
 }
 
@@ -461,8 +448,8 @@ static js_Ast *additive(js_State *J)
 {
 	js_Ast *a = multiplicative(J);
 loop:
-	if (accept(J, '+')) { a = EXP2(ADD, a, multiplicative(J)); goto loop; }
-	if (accept(J, '-')) { a = EXP2(SUB, a, multiplicative(J)); goto loop; }
+	if (jsP_accept(J, '+')) { a = EXP2(ADD, a, multiplicative(J)); goto loop; }
+	if (jsP_accept(J, '-')) { a = EXP2(SUB, a, multiplicative(J)); goto loop; }
 	return a;
 }
 
@@ -470,9 +457,9 @@ static js_Ast *shift(js_State *J)
 {
 	js_Ast *a = additive(J);
 loop:
-	if (accept(J, TK_SHL)) { a = EXP2(SHL, a, additive(J)); goto loop; }
-	if (accept(J, TK_SHR)) { a = EXP2(SHR, a, additive(J)); goto loop; }
-	if (accept(J, TK_USHR)) { a = EXP2(USHR, a, additive(J)); goto loop; }
+	if (jsP_accept(J, TK_SHL)) { a = EXP2(SHL, a, additive(J)); goto loop; }
+	if (jsP_accept(J, TK_SHR)) { a = EXP2(SHR, a, additive(J)); goto loop; }
+	if (jsP_accept(J, TK_USHR)) { a = EXP2(USHR, a, additive(J)); goto loop; }
 	return a;
 }
 
@@ -480,12 +467,12 @@ static js_Ast *relational(js_State *J, int notin)
 {
 	js_Ast *a = shift(J);
 loop:
-	if (accept(J, '<')) { a = EXP2(LT, a, shift(J)); goto loop; }
-	if (accept(J, '>')) { a = EXP2(GT, a, shift(J)); goto loop; }
-	if (accept(J, TK_LE)) { a = EXP2(LE, a, shift(J)); goto loop; }
-	if (accept(J, TK_GE)) { a = EXP2(GE, a, shift(J)); goto loop; }
-	if (accept(J, TK_INSTANCEOF)) { a = EXP2(INSTANCEOF, a, shift(J)); goto loop; }
-	if (!notin && accept(J, TK_IN)) { a = EXP2(IN, a, shift(J)); goto loop; }
+	if (jsP_accept(J, '<')) { a = EXP2(LT, a, shift(J)); goto loop; }
+	if (jsP_accept(J, '>')) { a = EXP2(GT, a, shift(J)); goto loop; }
+	if (jsP_accept(J, TK_LE)) { a = EXP2(LE, a, shift(J)); goto loop; }
+	if (jsP_accept(J, TK_GE)) { a = EXP2(GE, a, shift(J)); goto loop; }
+	if (jsP_accept(J, TK_INSTANCEOF)) { a = EXP2(INSTANCEOF, a, shift(J)); goto loop; }
+	if (!notin && jsP_accept(J, TK_IN)) { a = EXP2(IN, a, shift(J)); goto loop; }
 	return a;
 }
 
@@ -493,17 +480,17 @@ static js_Ast *equality(js_State *J, int notin)
 {
 	js_Ast *a = relational(J, notin);
 loop:
-	if (accept(J, TK_EQ)) { a = EXP2(EQ, a, relational(J, notin)); goto loop; }
-	if (accept(J, TK_NE)) { a = EXP2(NE, a, relational(J, notin)); goto loop; }
-	if (accept(J, TK_STRICTEQ)) { a = EXP2(STRICTEQ, a, relational(J, notin)); goto loop; }
-	if (accept(J, TK_STRICTNE)) { a = EXP2(STRICTNE, a, relational(J, notin)); goto loop; }
+	if (jsP_accept(J, TK_EQ)) { a = EXP2(EQ, a, relational(J, notin)); goto loop; }
+	if (jsP_accept(J, TK_NE)) { a = EXP2(NE, a, relational(J, notin)); goto loop; }
+	if (jsP_accept(J, TK_STRICTEQ)) { a = EXP2(STRICTEQ, a, relational(J, notin)); goto loop; }
+	if (jsP_accept(J, TK_STRICTNE)) { a = EXP2(STRICTNE, a, relational(J, notin)); goto loop; }
 	return a;
 }
 
 static js_Ast *bitand(js_State *J, int notin)
 {
 	js_Ast *a = equality(J, notin);
-	while (accept(J, '&'))
+	while (jsP_accept(J, '&'))
 		a = EXP2(BITAND, a, equality(J, notin));
 	return a;
 }
@@ -511,7 +498,7 @@ static js_Ast *bitand(js_State *J, int notin)
 static js_Ast *bitxor(js_State *J, int notin)
 {
 	js_Ast *a = bitand(J, notin);
-	while (accept(J, '^'))
+	while (jsP_accept(J, '^'))
 		a = EXP2(BITXOR, a, bitand(J, notin));
 	return a;
 }
@@ -519,7 +506,7 @@ static js_Ast *bitxor(js_State *J, int notin)
 static js_Ast *bitor(js_State *J, int notin)
 {
 	js_Ast *a = bitxor(J, notin);
-	while (accept(J, '|'))
+	while (jsP_accept(J, '|'))
 		a = EXP2(BITOR, a, bitxor(J, notin));
 	return a;
 }
@@ -527,7 +514,7 @@ static js_Ast *bitor(js_State *J, int notin)
 static js_Ast *logand(js_State *J, int notin)
 {
 	js_Ast *a = bitor(J, notin);
-	if (accept(J, TK_AND))
+	if (jsP_accept(J, TK_AND))
 		a = EXP2(LOGAND, a, logand(J, notin));
 	return a;
 }
@@ -535,7 +522,7 @@ static js_Ast *logand(js_State *J, int notin)
 static js_Ast *logor(js_State *J, int notin)
 {
 	js_Ast *a = logand(J, notin);
-	if (accept(J, TK_OR))
+	if (jsP_accept(J, TK_OR))
 		a = EXP2(LOGOR, a, logor(J, notin));
 	return a;
 }
@@ -544,9 +531,9 @@ static js_Ast *conditional(js_State *J, int notin)
 {
 	js_Ast *a, *b, *c;
 	a = logor(J, notin);
-	if (accept(J, '?')) {
+	if (jsP_accept(J, '?')) {
 		b = assignment(J, 0);
-		expect(J, ':');
+		jsP_expect(J, ':');
 		c = assignment(J, notin);
 		return EXP3(COND, a, b, c);
 	}
@@ -556,25 +543,25 @@ static js_Ast *conditional(js_State *J, int notin)
 static js_Ast *assignment(js_State *J, int notin)
 {
 	js_Ast *a = conditional(J, notin);
-	if (accept(J, '=')) return EXP2(ASS, a, assignment(J, notin));
-	if (accept(J, TK_MUL_ASS)) return EXP2(ASS_MUL, a, assignment(J, notin));
-	if (accept(J, TK_DIV_ASS)) return EXP2(ASS_DIV, a, assignment(J, notin));
-	if (accept(J, TK_MOD_ASS)) return EXP2(ASS_MOD, a, assignment(J, notin));
-	if (accept(J, TK_ADD_ASS)) return EXP2(ASS_ADD, a, assignment(J, notin));
-	if (accept(J, TK_SUB_ASS)) return EXP2(ASS_SUB, a, assignment(J, notin));
-	if (accept(J, TK_SHL_ASS)) return EXP2(ASS_SHL, a, assignment(J, notin));
-	if (accept(J, TK_SHR_ASS)) return EXP2(ASS_SHR, a, assignment(J, notin));
-	if (accept(J, TK_USHR_ASS)) return EXP2(ASS_USHR, a, assignment(J, notin));
-	if (accept(J, TK_AND_ASS)) return EXP2(ASS_BITAND, a, assignment(J, notin));
-	if (accept(J, TK_XOR_ASS)) return EXP2(ASS_BITXOR, a, assignment(J, notin));
-	if (accept(J, TK_OR_ASS)) return EXP2(ASS_BITOR, a, assignment(J, notin));
+	if (jsP_accept(J, '=')) return EXP2(ASS, a, assignment(J, notin));
+	if (jsP_accept(J, TK_MUL_ASS)) return EXP2(ASS_MUL, a, assignment(J, notin));
+	if (jsP_accept(J, TK_DIV_ASS)) return EXP2(ASS_DIV, a, assignment(J, notin));
+	if (jsP_accept(J, TK_MOD_ASS)) return EXP2(ASS_MOD, a, assignment(J, notin));
+	if (jsP_accept(J, TK_ADD_ASS)) return EXP2(ASS_ADD, a, assignment(J, notin));
+	if (jsP_accept(J, TK_SUB_ASS)) return EXP2(ASS_SUB, a, assignment(J, notin));
+	if (jsP_accept(J, TK_SHL_ASS)) return EXP2(ASS_SHL, a, assignment(J, notin));
+	if (jsP_accept(J, TK_SHR_ASS)) return EXP2(ASS_SHR, a, assignment(J, notin));
+	if (jsP_accept(J, TK_USHR_ASS)) return EXP2(ASS_USHR, a, assignment(J, notin));
+	if (jsP_accept(J, TK_AND_ASS)) return EXP2(ASS_BITAND, a, assignment(J, notin));
+	if (jsP_accept(J, TK_XOR_ASS)) return EXP2(ASS_BITXOR, a, assignment(J, notin));
+	if (jsP_accept(J, TK_OR_ASS)) return EXP2(ASS_BITOR, a, assignment(J, notin));
 	return a;
 }
 
 static js_Ast *expression(js_State *J, int notin)
 {
 	js_Ast *a = assignment(J, notin);
-	while (accept(J, ','))
+	while (jsP_accept(J, ','))
 		a = EXP2(COMMA, a, assignment(J, notin));
 	return a;
 }
@@ -584,7 +571,7 @@ static js_Ast *expression(js_State *J, int notin)
 static js_Ast *vardec(js_State *J, int notin)
 {
 	js_Ast *a = identifier(J);
-	if (accept(J, '='))
+	if (jsP_accept(J, '='))
 		return EXP2(VAR, a, assignment(J, notin));
 	return EXP1(VAR, a);
 }
@@ -593,7 +580,7 @@ static js_Ast *vardeclist(js_State *J, int notin)
 {
 	js_Ast *head, *tail;
 	head = tail = LIST(vardec(J, notin));
-	while (accept(J, ','))
+	while (jsP_accept(J, ','))
 		tail = tail->b = LIST(vardec(J, notin));
 	return jsP_list(head);
 }
@@ -613,20 +600,20 @@ static js_Ast *caseclause(js_State *J)
 {
 	js_Ast *a, *b;
 
-	if (accept(J, TK_CASE)) {
+	if (jsP_accept(J, TK_CASE)) {
 		a = expression(J, 0);
-		expect(J, ':');
+		jsP_expect(J, ':');
 		b = statementlist(J);
 		return STM2(CASE, a, b);
 	}
 
-	if (accept(J, TK_DEFAULT)) {
-		expect(J, ':');
+	if (jsP_accept(J, TK_DEFAULT)) {
+		jsP_expect(J, ':');
 		a = statementlist(J);
 		return STM1(DEFAULT, a);
 	}
 
-	jsP_error(J, "unexpected token in switch: %s (expected 'case' or 'default')", TOKSTR);
+	jsP_error(J, "unexpected token in switch: %s (expected 'case' or 'default')", jsY_tokenstring(J->lookahead));
 }
 
 static js_Ast *caselist(js_State *J)
@@ -643,9 +630,9 @@ static js_Ast *caselist(js_State *J)
 static js_Ast *block(js_State *J)
 {
 	js_Ast *a;
-	expect(J, '{');
+	jsP_expect(J, '{');
 	a = statementlist(J);
-	expect(J, '}');
+	jsP_expect(J, '}');
 	return STM1(BLOCK, a);
 }
 
@@ -654,48 +641,48 @@ static js_Ast *forexpression(js_State *J, int end)
 	js_Ast *a = NULL;
 	if (J->lookahead != end)
 		a = expression(J, 0);
-	expect(J, end);
+	jsP_expect(J, end);
 	return a;
 }
 
 static js_Ast *forstatement(js_State *J)
 {
 	js_Ast *a, *b, *c, *d;
-	expect(J, '(');
-	if (accept(J, TK_VAR)) {
+	jsP_expect(J, '(');
+	if (jsP_accept(J, TK_VAR)) {
 		a = vardeclist(J, 1);
-		if (accept(J, ';')) {
+		if (jsP_accept(J, ';')) {
 			b = forexpression(J, ';');
 			c = forexpression(J, ')');
 			d = statement(J);
 			return STM4(FOR_VAR, a, b, c, d);
 		}
-		if (accept(J, TK_IN)) {
+		if (jsP_accept(J, TK_IN)) {
 			b = expression(J, 0);
-			expect(J, ')');
+			jsP_expect(J, ')');
 			c = statement(J);
 			return STM3(FOR_IN_VAR, a, b, c);
 		}
-		jsP_error(J, "unexpected token in for-var-statement: %s", TOKSTR);
+		jsP_error(J, "unexpected token in for-var-statement: %s", jsY_tokenstring(J->lookahead));
 	}
 
 	if (J->lookahead != ';')
 		a = expression(J, 1);
 	else
 		a = NULL;
-	if (accept(J, ';')) {
+	if (jsP_accept(J, ';')) {
 		b = forexpression(J, ';');
 		c = forexpression(J, ')');
 		d = statement(J);
 		return STM4(FOR, a, b, c, d);
 	}
-	if (accept(J, TK_IN)) {
+	if (jsP_accept(J, TK_IN)) {
 		b = expression(J, 0);
-		expect(J, ')');
+		jsP_expect(J, ')');
 		c = statement(J);
 		return STM3(FOR_IN, a, b, c);
 	}
-	jsP_error(J, "unexpected token in for-statement: %s", TOKSTR);
+	jsP_error(J, "unexpected token in for-statement: %s", jsY_tokenstring(J->lookahead));
 }
 
 static js_Ast *statement(js_State *J)
@@ -706,64 +693,64 @@ static js_Ast *statement(js_State *J)
 		return block(J);
 	}
 
-	if (accept(J, TK_VAR)) {
+	if (jsP_accept(J, TK_VAR)) {
 		a = vardeclist(J, 0);
 		semicolon(J);
 		return STM1(VAR, a);
 	}
 
 	/* empty statement */
-	if (accept(J, ';')) {
+	if (jsP_accept(J, ';')) {
 		return STM0(EMPTY);
 	}
 
-	if (accept(J, TK_IF)) {
-		expect(J, '(');
+	if (jsP_accept(J, TK_IF)) {
+		jsP_expect(J, '(');
 		a = expression(J, 0);
-		expect(J, ')');
+		jsP_expect(J, ')');
 		b = statement(J);
-		if (accept(J, TK_ELSE))
+		if (jsP_accept(J, TK_ELSE))
 			c = statement(J);
 		else
 			c = NULL;
 		return STM3(IF, a, b, c);
 	}
 
-	if (accept(J, TK_DO)) {
+	if (jsP_accept(J, TK_DO)) {
 		a = statement(J);
-		expect(J, TK_WHILE);
-		expect(J, '(');
+		jsP_expect(J, TK_WHILE);
+		jsP_expect(J, '(');
 		b = expression(J, 0);
-		expect(J, ')');
+		jsP_expect(J, ')');
 		semicolon(J);
 		return STM2(DO, a, b);
 	}
 
-	if (accept(J, TK_WHILE)) {
-		expect(J, '(');
+	if (jsP_accept(J, TK_WHILE)) {
+		jsP_expect(J, '(');
 		a = expression(J, 0);
-		expect(J, ')');
+		jsP_expect(J, ')');
 		b = statement(J);
 		return STM2(WHILE, a, b);
 	}
 
-	if (accept(J, TK_FOR)) {
+	if (jsP_accept(J, TK_FOR)) {
 		return forstatement(J);
 	}
 
-	if (accept(J, TK_CONTINUE)) {
+	if (jsP_accept(J, TK_CONTINUE)) {
 		a = identifieropt(J);
 		semicolon(J);
 		return STM1(CONTINUE, a);
 	}
 
-	if (accept(J, TK_BREAK)) {
+	if (jsP_accept(J, TK_BREAK)) {
 		a = identifieropt(J);
 		semicolon(J);
 		return STM1(BREAK, a);
 	}
 
-	if (accept(J, TK_RETURN)) {
+	if (jsP_accept(J, TK_RETURN)) {
 		if (J->lookahead != ';' && J->lookahead != '}' && J->lookahead != 0)
 			a = expression(J, 0);
 		else
@@ -772,53 +759,55 @@ static js_Ast *statement(js_State *J)
 		return STM1(RETURN, a);
 	}
 
-	if (accept(J, TK_WITH)) {
-		expect(J, '(');
+	if (jsP_accept(J, TK_WITH)) {
+		if (J->strict)
+			jsP_error(J, "'with' statements are not allowed in strict mode");
+		jsP_expect(J, '(');
 		a = expression(J, 0);
-		expect(J, ')');
+		jsP_expect(J, ')');
 		b = statement(J);
 		return STM2(WITH, a, b);
 	}
 
-	if (accept(J, TK_SWITCH)) {
-		expect(J, '(');
+	if (jsP_accept(J, TK_SWITCH)) {
+		jsP_expect(J, '(');
 		a = expression(J, 0);
-		expect(J, ')');
-		expect(J, '{');
+		jsP_expect(J, ')');
+		jsP_expect(J, '{');
 		b = caselist(J);
-		expect(J, '}');
+		jsP_expect(J, '}');
 		return STM2(SWITCH, a, b);
 	}
 
-	if (accept(J, TK_THROW)) {
+	if (jsP_accept(J, TK_THROW)) {
 		a = expression(J, 0);
 		semicolon(J);
 		return STM1(THROW, a);
 	}
 
-	if (accept(J, TK_TRY)) {
+	if (jsP_accept(J, TK_TRY)) {
 		a = block(J);
 		b = c = d = NULL;
-		if (accept(J, TK_CATCH)) {
-			expect(J, '(');
+		if (jsP_accept(J, TK_CATCH)) {
+			jsP_expect(J, '(');
 			b = identifier(J);
-			expect(J, ')');
+			jsP_expect(J, ')');
 			c = block(J);
 		}
-		if (accept(J, TK_FINALLY)) {
+		if (jsP_accept(J, TK_FINALLY)) {
 			d = block(J);
 		}
 		if (!b && !d)
-			jsP_error(J, "unexpected token in try: %s (expected 'catch' or 'finally')", TOKSTR);
+			jsP_error(J, "unexpected token in try: %s (expected 'catch' or 'finally')", jsY_tokenstring(J->lookahead));
 		return STM4(TRY, a, b, c, d);
 	}
 
-	if (accept(J, TK_DEBUGGER)) {
+	if (jsP_accept(J, TK_DEBUGGER)) {
 		semicolon(J);
 		return STM0(DEBUGGER);
 	}
 
-	if (accept(J, TK_FUNCTION)) {
+	if (jsP_accept(J, TK_FUNCTION)) {
 		jsP_warning(J, "function statements are not standard");
 		return funstm(J);
 	}
@@ -826,7 +815,7 @@ static js_Ast *statement(js_State *J)
 	/* labelled statement or expression statement */
 	if (J->lookahead == TK_IDENTIFIER) {
 		a = expression(J, 0);
-		if (a->type == EXP_IDENTIFIER && accept(J, ':')) {
+		if (a->type == EXP_IDENTIFIER && jsP_accept(J, ':')) {
 			a->type = AST_IDENTIFIER;
 			b = statement(J);
 			return STM2(LABEL, a, b);
@@ -845,7 +834,7 @@ static js_Ast *statement(js_State *J)
 
 static js_Ast *scriptelement(js_State *J)
 {
-	if (accept(J, TK_FUNCTION))
+	if (jsP_accept(J, TK_FUNCTION))
 		return fundec(J);
 	return statement(J);
 }
@@ -864,9 +853,9 @@ static js_Ast *script(js_State *J, int terminator)
 static js_Ast *funbody(js_State *J)
 {
 	js_Ast *a;
-	expect(J, '{');
+	jsP_expect(J, '{');
 	a = script(J, '}');
-	expect(J, '}');
+	jsP_expect(J, '}');
 	return a;
 }
 
@@ -950,7 +939,7 @@ js_Ast *jsP_parse(js_State *J, const char *filename, const char *source)
 	js_Ast *p;
 
 	jsY_initlex(J, filename, source);
-	next(J);
+	jsP_next(J);
 	p = script(J, 0);
 	if (p)
 		jsP_foldconst(p);
@@ -963,7 +952,7 @@ js_Ast *jsP_parsefunction(js_State *J, const char *filename, const char *params,
 	js_Ast *p = NULL;
 	if (params) {
 		jsY_initlex(J, filename, params);
-		next(J);
+		jsP_next(J);
 		p = parameters(J);
 	}
 	return EXP3(FUN, NULL, p, jsP_parse(J, filename, body));
