@@ -6,7 +6,8 @@
 "rot4",
 "number_0",
 "number_1",
-"number_n",
+"number_pos",
+"number_neg",
 "number",
 "string",
 "closure",
@@ -32,8 +33,6 @@
 "delvar",
 "in",
 "initprop",
-"initprop_n",
-"initprop_s",
 "initgetter",
 "initsetter",
 "getprop",
@@ -44,6 +43,7 @@
 "delprop_s",
 "iterator",
 "nextiter",
+"eval",
 "call",
 "new",
 "typeof",
@@ -88,3 +88,4 @@
 "jtrue",
 "jfalse",
 "return",
+"line",
